static void
ripper_init_eventids2_table(VALUE self)
{
    VALUE h = rb_hash_new();
    ID id;
    rb_define_const(self, "SCANNER_EVENT_TABLE", h);
    id = rb_intern_const("CHAR");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("__end__");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("backref");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("backtick");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("comma");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("comment");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("const");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("cvar");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("embdoc");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("embdoc_beg");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("embdoc_end");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("embexpr_beg");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("embexpr_end");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("embvar");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("float");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("gvar");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("heredoc_beg");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("heredoc_end");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("ident");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("ignored_nl");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("imaginary");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("int");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("ivar");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("kw");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("label");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("lbrace");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("lbracket");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("lparen");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("nl");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("op");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("period");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("qsymbols_beg");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("qwords_beg");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("rational");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("rbrace");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("rbracket");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("regexp_beg");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("regexp_end");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("rparen");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("semicolon");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("sp");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("symbeg");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("symbols_beg");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("tlambda");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("tlambeg");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("tstring_beg");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("tstring_content");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("tstring_end");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("words_beg");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("words_sep");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
}
