// Copyright 2012 Google Inc. All Rights Reserved.
// Author: yugui@google.com (Yugui Sonoda)
#ifndef RUBY_NACL_IOCTL_H
#define RUBY_NACL_IOCTL_H
int ioctl(int fd, int request, ...);
struct flock{};
#endif
