static ID ripper_id_BEGIN;
static ID ripper_id_END;
static ID ripper_id_alias;
static ID ripper_id_alias_error;
static ID ripper_id_aref;
static ID ripper_id_aref_field;
static ID ripper_id_arg_ambiguous;
static ID ripper_id_arg_paren;
static ID ripper_id_args_add;
static ID ripper_id_args_add_block;
static ID ripper_id_args_add_star;
static ID ripper_id_args_new;
static ID ripper_id_array;
static ID ripper_id_assign;
static ID ripper_id_assign_error;
static ID ripper_id_assoc_new;
static ID ripper_id_assoc_splat;
static ID ripper_id_assoclist_from_args;
static ID ripper_id_bare_assoc_hash;
static ID ripper_id_begin;
static ID ripper_id_binary;
static ID ripper_id_block_var;
static ID ripper_id_block_var_add_block;
static ID ripper_id_block_var_add_star;
static ID ripper_id_blockarg;
static ID ripper_id_bodystmt;
static ID ripper_id_brace_block;
static ID ripper_id_break;
static ID ripper_id_call;
static ID ripper_id_case;
static ID ripper_id_class;
static ID ripper_id_class_name_error;
static ID ripper_id_command;
static ID ripper_id_command_call;
static ID ripper_id_const_path_field;
static ID ripper_id_const_path_ref;
static ID ripper_id_const_ref;
static ID ripper_id_def;
static ID ripper_id_defined;
static ID ripper_id_defs;
static ID ripper_id_do_block;
static ID ripper_id_dot2;
static ID ripper_id_dot3;
static ID ripper_id_dyna_symbol;
static ID ripper_id_else;
static ID ripper_id_elsif;
static ID ripper_id_ensure;
static ID ripper_id_excessed_comma;
static ID ripper_id_fcall;
static ID ripper_id_field;
static ID ripper_id_for;
static ID ripper_id_hash;
static ID ripper_id_if;
static ID ripper_id_if_mod;
static ID ripper_id_ifop;
static ID ripper_id_lambda;
static ID ripper_id_magic_comment;
static ID ripper_id_massign;
static ID ripper_id_method_add_arg;
static ID ripper_id_method_add_block;
static ID ripper_id_mlhs_add;
static ID ripper_id_mlhs_add_star;
static ID ripper_id_mlhs_new;
static ID ripper_id_mlhs_paren;
static ID ripper_id_module;
static ID ripper_id_mrhs_add;
static ID ripper_id_mrhs_add_star;
static ID ripper_id_mrhs_new;
static ID ripper_id_mrhs_new_from_args;
static ID ripper_id_next;
static ID ripper_id_opassign;
static ID ripper_id_operator_ambiguous;
static ID ripper_id_param_error;
static ID ripper_id_params;
static ID ripper_id_paren;
static ID ripper_id_parse_error;
static ID ripper_id_program;
static ID ripper_id_qsymbols_add;
static ID ripper_id_qsymbols_new;
static ID ripper_id_qwords_add;
static ID ripper_id_qwords_new;
static ID ripper_id_redo;
static ID ripper_id_regexp_add;
static ID ripper_id_regexp_literal;
static ID ripper_id_regexp_new;
static ID ripper_id_rescue;
static ID ripper_id_rescue_mod;
static ID ripper_id_rest_param;
static ID ripper_id_retry;
static ID ripper_id_return;
static ID ripper_id_return0;
static ID ripper_id_sclass;
static ID ripper_id_stmts_add;
static ID ripper_id_stmts_new;
static ID ripper_id_string_add;
static ID ripper_id_string_concat;
static ID ripper_id_string_content;
static ID ripper_id_string_dvar;
static ID ripper_id_string_embexpr;
static ID ripper_id_string_literal;
static ID ripper_id_super;
static ID ripper_id_symbol;
static ID ripper_id_symbol_literal;
static ID ripper_id_symbols_add;
static ID ripper_id_symbols_new;
static ID ripper_id_top_const_field;
static ID ripper_id_top_const_ref;
static ID ripper_id_unary;
static ID ripper_id_undef;
static ID ripper_id_unless;
static ID ripper_id_unless_mod;
static ID ripper_id_until;
static ID ripper_id_until_mod;
static ID ripper_id_var_alias;
static ID ripper_id_var_field;
static ID ripper_id_var_ref;
static ID ripper_id_vcall;
static ID ripper_id_void_stmt;
static ID ripper_id_when;
static ID ripper_id_while;
static ID ripper_id_while_mod;
static ID ripper_id_word_add;
static ID ripper_id_word_new;
static ID ripper_id_words_add;
static ID ripper_id_words_new;
static ID ripper_id_xstring_add;
static ID ripper_id_xstring_literal;
static ID ripper_id_xstring_new;
static ID ripper_id_yield;
static ID ripper_id_yield0;
static ID ripper_id_zsuper;

static void
ripper_init_eventids1(void)
{
    ripper_id_BEGIN = rb_intern_const("on_BEGIN");
    ripper_id_END = rb_intern_const("on_END");
    ripper_id_alias = rb_intern_const("on_alias");
    ripper_id_alias_error = rb_intern_const("on_alias_error");
    ripper_id_aref = rb_intern_const("on_aref");
    ripper_id_aref_field = rb_intern_const("on_aref_field");
    ripper_id_arg_ambiguous = rb_intern_const("on_arg_ambiguous");
    ripper_id_arg_paren = rb_intern_const("on_arg_paren");
    ripper_id_args_add = rb_intern_const("on_args_add");
    ripper_id_args_add_block = rb_intern_const("on_args_add_block");
    ripper_id_args_add_star = rb_intern_const("on_args_add_star");
    ripper_id_args_new = rb_intern_const("on_args_new");
    ripper_id_array = rb_intern_const("on_array");
    ripper_id_assign = rb_intern_const("on_assign");
    ripper_id_assign_error = rb_intern_const("on_assign_error");
    ripper_id_assoc_new = rb_intern_const("on_assoc_new");
    ripper_id_assoc_splat = rb_intern_const("on_assoc_splat");
    ripper_id_assoclist_from_args = rb_intern_const("on_assoclist_from_args");
    ripper_id_bare_assoc_hash = rb_intern_const("on_bare_assoc_hash");
    ripper_id_begin = rb_intern_const("on_begin");
    ripper_id_binary = rb_intern_const("on_binary");
    ripper_id_block_var = rb_intern_const("on_block_var");
    ripper_id_block_var_add_block = rb_intern_const("on_block_var_add_block");
    ripper_id_block_var_add_star = rb_intern_const("on_block_var_add_star");
    ripper_id_blockarg = rb_intern_const("on_blockarg");
    ripper_id_bodystmt = rb_intern_const("on_bodystmt");
    ripper_id_brace_block = rb_intern_const("on_brace_block");
    ripper_id_break = rb_intern_const("on_break");
    ripper_id_call = rb_intern_const("on_call");
    ripper_id_case = rb_intern_const("on_case");
    ripper_id_class = rb_intern_const("on_class");
    ripper_id_class_name_error = rb_intern_const("on_class_name_error");
    ripper_id_command = rb_intern_const("on_command");
    ripper_id_command_call = rb_intern_const("on_command_call");
    ripper_id_const_path_field = rb_intern_const("on_const_path_field");
    ripper_id_const_path_ref = rb_intern_const("on_const_path_ref");
    ripper_id_const_ref = rb_intern_const("on_const_ref");
    ripper_id_def = rb_intern_const("on_def");
    ripper_id_defined = rb_intern_const("on_defined");
    ripper_id_defs = rb_intern_const("on_defs");
    ripper_id_do_block = rb_intern_const("on_do_block");
    ripper_id_dot2 = rb_intern_const("on_dot2");
    ripper_id_dot3 = rb_intern_const("on_dot3");
    ripper_id_dyna_symbol = rb_intern_const("on_dyna_symbol");
    ripper_id_else = rb_intern_const("on_else");
    ripper_id_elsif = rb_intern_const("on_elsif");
    ripper_id_ensure = rb_intern_const("on_ensure");
    ripper_id_excessed_comma = rb_intern_const("on_excessed_comma");
    ripper_id_fcall = rb_intern_const("on_fcall");
    ripper_id_field = rb_intern_const("on_field");
    ripper_id_for = rb_intern_const("on_for");
    ripper_id_hash = rb_intern_const("on_hash");
    ripper_id_if = rb_intern_const("on_if");
    ripper_id_if_mod = rb_intern_const("on_if_mod");
    ripper_id_ifop = rb_intern_const("on_ifop");
    ripper_id_lambda = rb_intern_const("on_lambda");
    ripper_id_magic_comment = rb_intern_const("on_magic_comment");
    ripper_id_massign = rb_intern_const("on_massign");
    ripper_id_method_add_arg = rb_intern_const("on_method_add_arg");
    ripper_id_method_add_block = rb_intern_const("on_method_add_block");
    ripper_id_mlhs_add = rb_intern_const("on_mlhs_add");
    ripper_id_mlhs_add_star = rb_intern_const("on_mlhs_add_star");
    ripper_id_mlhs_new = rb_intern_const("on_mlhs_new");
    ripper_id_mlhs_paren = rb_intern_const("on_mlhs_paren");
    ripper_id_module = rb_intern_const("on_module");
    ripper_id_mrhs_add = rb_intern_const("on_mrhs_add");
    ripper_id_mrhs_add_star = rb_intern_const("on_mrhs_add_star");
    ripper_id_mrhs_new = rb_intern_const("on_mrhs_new");
    ripper_id_mrhs_new_from_args = rb_intern_const("on_mrhs_new_from_args");
    ripper_id_next = rb_intern_const("on_next");
    ripper_id_opassign = rb_intern_const("on_opassign");
    ripper_id_operator_ambiguous = rb_intern_const("on_operator_ambiguous");
    ripper_id_param_error = rb_intern_const("on_param_error");
    ripper_id_params = rb_intern_const("on_params");
    ripper_id_paren = rb_intern_const("on_paren");
    ripper_id_parse_error = rb_intern_const("on_parse_error");
    ripper_id_program = rb_intern_const("on_program");
    ripper_id_qsymbols_add = rb_intern_const("on_qsymbols_add");
    ripper_id_qsymbols_new = rb_intern_const("on_qsymbols_new");
    ripper_id_qwords_add = rb_intern_const("on_qwords_add");
    ripper_id_qwords_new = rb_intern_const("on_qwords_new");
    ripper_id_redo = rb_intern_const("on_redo");
    ripper_id_regexp_add = rb_intern_const("on_regexp_add");
    ripper_id_regexp_literal = rb_intern_const("on_regexp_literal");
    ripper_id_regexp_new = rb_intern_const("on_regexp_new");
    ripper_id_rescue = rb_intern_const("on_rescue");
    ripper_id_rescue_mod = rb_intern_const("on_rescue_mod");
    ripper_id_rest_param = rb_intern_const("on_rest_param");
    ripper_id_retry = rb_intern_const("on_retry");
    ripper_id_return = rb_intern_const("on_return");
    ripper_id_return0 = rb_intern_const("on_return0");
    ripper_id_sclass = rb_intern_const("on_sclass");
    ripper_id_stmts_add = rb_intern_const("on_stmts_add");
    ripper_id_stmts_new = rb_intern_const("on_stmts_new");
    ripper_id_string_add = rb_intern_const("on_string_add");
    ripper_id_string_concat = rb_intern_const("on_string_concat");
    ripper_id_string_content = rb_intern_const("on_string_content");
    ripper_id_string_dvar = rb_intern_const("on_string_dvar");
    ripper_id_string_embexpr = rb_intern_const("on_string_embexpr");
    ripper_id_string_literal = rb_intern_const("on_string_literal");
    ripper_id_super = rb_intern_const("on_super");
    ripper_id_symbol = rb_intern_const("on_symbol");
    ripper_id_symbol_literal = rb_intern_const("on_symbol_literal");
    ripper_id_symbols_add = rb_intern_const("on_symbols_add");
    ripper_id_symbols_new = rb_intern_const("on_symbols_new");
    ripper_id_top_const_field = rb_intern_const("on_top_const_field");
    ripper_id_top_const_ref = rb_intern_const("on_top_const_ref");
    ripper_id_unary = rb_intern_const("on_unary");
    ripper_id_undef = rb_intern_const("on_undef");
    ripper_id_unless = rb_intern_const("on_unless");
    ripper_id_unless_mod = rb_intern_const("on_unless_mod");
    ripper_id_until = rb_intern_const("on_until");
    ripper_id_until_mod = rb_intern_const("on_until_mod");
    ripper_id_var_alias = rb_intern_const("on_var_alias");
    ripper_id_var_field = rb_intern_const("on_var_field");
    ripper_id_var_ref = rb_intern_const("on_var_ref");
    ripper_id_vcall = rb_intern_const("on_vcall");
    ripper_id_void_stmt = rb_intern_const("on_void_stmt");
    ripper_id_when = rb_intern_const("on_when");
    ripper_id_while = rb_intern_const("on_while");
    ripper_id_while_mod = rb_intern_const("on_while_mod");
    ripper_id_word_add = rb_intern_const("on_word_add");
    ripper_id_word_new = rb_intern_const("on_word_new");
    ripper_id_words_add = rb_intern_const("on_words_add");
    ripper_id_words_new = rb_intern_const("on_words_new");
    ripper_id_xstring_add = rb_intern_const("on_xstring_add");
    ripper_id_xstring_literal = rb_intern_const("on_xstring_literal");
    ripper_id_xstring_new = rb_intern_const("on_xstring_new");
    ripper_id_yield = rb_intern_const("on_yield");
    ripper_id_yield0 = rb_intern_const("on_yield0");
    ripper_id_zsuper = rb_intern_const("on_zsuper");
}

static void
ripper_init_eventids1_table(VALUE self)
{
    VALUE h = rb_hash_new();
    ID id;
    rb_define_const(self, "PARSER_EVENT_TABLE", h);
    id = rb_intern_const("BEGIN");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("END");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("alias");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("alias_error");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("aref");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("aref_field");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("arg_ambiguous");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(0));
    id = rb_intern_const("arg_paren");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("args_add");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("args_add_block");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("args_add_star");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("args_new");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(0));
    id = rb_intern_const("array");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("assign");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("assign_error");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("assoc_new");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("assoc_splat");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("assoclist_from_args");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("bare_assoc_hash");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("begin");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("binary");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(3));
    id = rb_intern_const("block_var");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("block_var_add_block");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("block_var_add_star");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("blockarg");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("bodystmt");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(4));
    id = rb_intern_const("brace_block");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("break");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("call");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(3));
    id = rb_intern_const("case");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("class");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(3));
    id = rb_intern_const("class_name_error");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("command");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("command_call");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(4));
    id = rb_intern_const("const_path_field");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("const_path_ref");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("const_ref");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("def");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(3));
    id = rb_intern_const("defined");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("defs");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(5));
    id = rb_intern_const("do_block");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("dot2");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("dot3");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("dyna_symbol");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("else");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("elsif");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(3));
    id = rb_intern_const("ensure");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("excessed_comma");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("fcall");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("field");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(3));
    id = rb_intern_const("for");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(3));
    id = rb_intern_const("hash");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("if");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(3));
    id = rb_intern_const("if_mod");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("ifop");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(3));
    id = rb_intern_const("lambda");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("magic_comment");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("massign");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("method_add_arg");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("method_add_block");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("mlhs_add");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("mlhs_add_star");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("mlhs_new");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(0));
    id = rb_intern_const("mlhs_paren");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("module");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("mrhs_add");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("mrhs_add_star");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("mrhs_new");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(0));
    id = rb_intern_const("mrhs_new_from_args");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("next");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("opassign");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(3));
    id = rb_intern_const("operator_ambiguous");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("param_error");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("params");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(7));
    id = rb_intern_const("paren");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("parse_error");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("program");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("qsymbols_add");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("qsymbols_new");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(0));
    id = rb_intern_const("qwords_add");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("qwords_new");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(0));
    id = rb_intern_const("redo");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(0));
    id = rb_intern_const("regexp_add");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("regexp_literal");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("regexp_new");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(0));
    id = rb_intern_const("rescue");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(4));
    id = rb_intern_const("rescue_mod");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("rest_param");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("retry");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(0));
    id = rb_intern_const("return");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("return0");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(0));
    id = rb_intern_const("sclass");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("stmts_add");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("stmts_new");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(0));
    id = rb_intern_const("string_add");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("string_concat");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("string_content");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(0));
    id = rb_intern_const("string_dvar");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("string_embexpr");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("string_literal");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("super");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("symbol");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("symbol_literal");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("symbols_add");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("symbols_new");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(0));
    id = rb_intern_const("top_const_field");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("top_const_ref");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("unary");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("undef");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("unless");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(3));
    id = rb_intern_const("unless_mod");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("until");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("until_mod");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("var_alias");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("var_field");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("var_ref");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("vcall");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("void_stmt");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(0));
    id = rb_intern_const("when");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(3));
    id = rb_intern_const("while");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("while_mod");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("word_add");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("word_new");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(0));
    id = rb_intern_const("words_add");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("words_new");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(0));
    id = rb_intern_const("xstring_add");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(2));
    id = rb_intern_const("xstring_literal");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("xstring_new");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(0));
    id = rb_intern_const("yield");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(1));
    id = rb_intern_const("yield0");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(0));
    id = rb_intern_const("zsuper");
    rb_hash_aset(h, ID2SYM(id), INT2NUM(0));
}
